﻿DELIMITER $$


DROP PROCEDURE IF EXISTS `prune_oo_data` $$
CREATE DEFINER=`dharma_user`@`localhost` PROCEDURE `prune_oo_data`(hoursToKeep int
                               , prune_batch_size int
                               , prune_dashboards varchar(5)
                               , verbose int
                                 )
    MODIFIES SQL DATA
BEGIN

  declare lastRunDate datetime;
  declare deleteOlderThan datetime;

  declare deleteFromIndex int;
  declare deleteToIndex int;
  declare lastPruneTableIndex int;

  SELECT max(start_time)
  INTO lastRunDate
  FROM run_history;


  IF verbose > 0 THEN
    SELECT CONCAT('Last entry in the run_history table occured on ', lastRunDate) AS "INFO";
  END IF;


  SET hoursToKeep = hoursToKeep * -1;
  SET deleteOlderThan = TIMESTAMPADD(HOUR, hoursToKeep, lastRunDate);


  IF verbose > 0 THEN
    SELECT CONCAT('Deleting entries older than ', deleteOlderThan) AS "INFO";
  END IF;

  -- drop the temp table just in case it's still around from a failed run
  DROP TEMPORARY TABLE IF EXISTS oo_prune_table;
  CREATE TEMPORARY TABLE oo_prune_table
         (
            `oid` bigint(20) NOT NULL AUTO_INCREMENT,
            `run_hist_id` bigint(20) NOT NULL,
            `execution_state` int(11) DEFAULT NULL,
            `flow_uuid` varchar(255) DEFAULT NULL,
            `flow_version` bigint(20) DEFAULT NULL,
            `run_time_millis` bigint(20) DEFAULT NULL,
            PRIMARY KEY (`oid`),
            KEY `idx_oo_prune_table_run_hist_id` (`run_hist_id`)
         )
         ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;


  -- get the info for the records to delete, making sure not to remove anything that's still
  -- in the run table.
  INSERT INTO oo_prune_table(run_hist_id, execution_state, flow_uuid, flow_version, run_time_millis)
  SELECT      oid, execution_state, flow_uuid, flow_version, CAST(run_time_millis as UNSIGNED)
  FROM        run_history AS h
  WHERE       (  start_time < deleteOlderThan
                  AND
                 oid NOT IN (SELECT history_id FROM run)
              )
  ORDER BY h.oid ASC;


  -- get the first and last indexes from oo_prune_table
  SELECT min(oid), max(oid)
  INTO deleteFromIndex, lastPruneTableIndex
  FROM oo_prune_table;


  -- loop through oo_prune_table, stepping by batch_size, and delete the data
  IF verbose > 0 THEN
    SELECT CONCAT('Pruning ', lastPruneTableIndex - deleteFromIndex,
                  ' from the database. This may take a while...') AS "INFO";
  END IF;

  WHILE deleteFromIndex <= lastPruneTableIndex DO

    -- calculate the end of the delete range
    SET deleteToIndex = deleteFromIndex + prune_batch_size;

    -- on the off chance that a batch ends on the end of the batch table,
    -- we would end up in an infinite loop without this check.
    IF deleteToIndex = deleteFromIndex THEN
      SET deleteFromIndex = deleteFromIndex + 1;
    END IF;



    IF verbose > 1 THEN
      SET @msg = CONCAT('Deleting chunk: ',
                        deleteFromIndex, ' to ',
                        deleteToIndex);
      SELECT @msg as "INFO";
    END IF;

    START TRANSACTION;

      -- delete dashboard data from log_record if requested
      IF (LOWER(prune_dashboards) = 'true') THEN

        IF (verbose > 1) THEN
          SELECT 'Deleting dashboard data...' AS "INFO";
        END IF;

        DELETE l
        FROM log_record l
          INNER JOIN oo_prune_table p
                  ON ((p.oid BETWEEN deleteFromIndex AND deleteToIndex)
                  AND (l.run_hist_id = p.run_hist_id));

      ELSE
        -- notify the user that we're not deleting dashboards
        IF (verbose > 1) THEN
          SELECT 'Not deleting dashboard data...' AS "INFO";
        END IF;

      END IF;


      IF (verbose > 1) THEN
        SELECT 'Deleting run history data...' AS "INFO";
      END IF;

      DELETE r
      FROM run_history AS r
      INNER JOIN oo_prune_table as p
            ON
              p.oid >= deleteFromIndex AND
              p.oid < deleteToIndex    AND
              r.oid = p.run_hist_id;


      IF (verbose > 1) THEN
        SELECT 'Updating flow metrics...' AS "INFO";
        SELECT 'BEFORE:' AS "INFO";
        SELECT * FROM flow_metrics;

      END IF;


      UPDATE flow_metrics AS f
        INNER JOIN (SELECT flow_uuid,
                           flow_version,
                           sum(if(execution_state = 0, 1, 0)) AS diagnosedCount,
                           sum(if(execution_state = 1, 1, 0)) AS resolvedCount,
                           sum(if(execution_state = 2, 1, 0)) AS noActionCount,
                           sum(IF(execution_state = 3, 1, 0)) AS errorCount,
                           sum(IF(execution_state = 2147483647, 1, 0)) AS failedCount,
                           sum(run_time_millis) AS cumulativeTime
                    FROM oo_prune_table
                    WHERE oid BETWEEN deleteFromIndex AND deleteToIndex
                    GROUP BY flow_uuid, flow_version
                   ) AS d
        ON f.flow_uuid = d.flow_uuid AND f.flow_version = d.flow_version
      SET f.diagnosed_count = f.diagnosed_count - d.diagnosedCount,
          f.resolved_count = f.resolved_count - d.resolvedCount,
          f.failed_count = f.failed_count - d.failedCount,
          f.no_action_count = f.no_action_count - d.noActionCount,
          f.resolved_count = f.resolved_count - d.resolvedCount,
          f.cumulative_time = f.cumulative_time - d.cumulativeTime,
          f.dlm_time = NOW();

      -- now delete the metrics for those flows that are
      -- left with 0 counts across the board
      DELETE FROM flow_metrics
      WHERE diagnosed_count = 0
        AND failed_count = 0
        AND no_action_count = 0
        AND resolved_count = 0
        AND error_count = 0
        AND EXISTS (SELECT 1 FROM oo_prune_table p
                    WHERE oid BETWEEN deleteFromIndex AND deleteToIndex
                      AND flow_uuid = p.flow_uuid);


      IF (verbose > 1) THEN
        SELECT 'AFTER:' AS "INFO";
        SELECT * FROM flow_metrics;

      END IF;


    COMMIT;

    -- move the start index to start one after the last index deleted.
    SET deleteFromIndex = deleteToIndex;

  END WHILE;

  IF verbose > 0 THEN
    SELECT 'Pruning complete.' AS "INFO";
  END IF;

  -- drop the temp table to free up resources.
  DROP TEMPORARY TABLE oo_prune_table;

END $$

DELIMITER ;